package logique;

import com.raylib.Jaylib;
import com.raylib.Raylib;
import utils.Pair;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.List;

public class Population {

    private int chromosomeLength = 0;
    private BitSet chromosome;
    private int posX = 0;
    private int posY = 4;

    public Population() {
        this.chromosome = new BitSet();
    }

    public Population(BitSet chromosome) {
        this.chromosome = chromosome;
    }

    public void move(int type) {
        int movement = 0;
        switch (type) {
            case 0: // UP
                movement = -1;
                if (isCoordinateValid(posX, posY + movement)) {
                    posY += movement;
                }
                break;
            case 1: // DOWN
                movement = 1;
                if (isCoordinateValid(posX, posY + movement)) {
                    posY += movement;
                }
                break;
            case 2: // L
                movement = -1;
                if (isCoordinateValid(posX + movement, posY)) {
                    posX += movement;
                }
                break;
            case 3: // R
                movement = 1;
                if (isCoordinateValid(posX + movement, posY)) {
                    posX += movement;
                }
                break;
        }

        int[] bin = new int[2];
        for (int i = 0; type > 0; i++) {
            bin[i] = type % 2;
            type /= 2;
        }

        for (int j : bin) {
            chromosome.set(chromosomeLength++, j == 1);
        }
    }

    public float score() {
        return (float) Math.pow((double) Maze.goal.getFirst() - posX, 2.d) +
                (float) Math.pow((double) Maze.goal.getSecond() - posY, 2.d);
    }

    public List<Integer> getMoves() {
        List<Integer> moves = new ArrayList<>();
        for (int i = 0; i < chromosomeLength; i += 2) {
            String s = "";
            for (int j = i; j < i+2; j++) {
                s += chromosome.get(j) ? "1" : "0";
            }
            moves.add(Integer.parseInt(s, 2));
        }

        return moves;
    }

    private boolean isCoordinateValid(int x, int y) {
        if (x > Maze.MAZE[0].length || x < 0) return false;
        if (y > Maze.MAZE.length || y < 0) return false;
        return Maze.MAZE[y][x] != 1;
    }

    public void render() {
        Raylib.DrawCircle(posX * 50 + 25, posY * 50 + 25, 25.f, Jaylib.RED);
    }

    public Pair<Integer, Integer> getCoordinates() {
        return new Pair<>(posX, posY);
    }

    public void setCoordinates(int x, int y) {
        posX = x;
        posY = y;
    }

    public BitSet getChromosome() {
        return chromosome;
    }

    public void setChromosome(BitSet chromosome) {
        this.chromosome = chromosome;
    }
}
